<?php

/**
 * @file
 * The Welcoming Websites Wizard installation profile
 */

/**
 * Implementation of hook_profile_modules.
 */
function wwwizard_profile_details() {
  return array(
    'name' => st('Welcoming Websites Wizard'),
    'description' => st('An installation profile for building websites for Unitarian Universalist congregations.'),
  );
}

/**
 * Implementation of hook_profile_modules.
 */
function wwwizard_profile_modules() {
  $modules = array(
    // required core modules
    'block', 'filter', 'node', 'system', 'user', 

    // optional core modules
    'color', 'dblog', 'help', 'menu', 'path', 'taxonomy', 'update', 'blog', 'search',

    // installed modules
    // admin
    'advanced_help', 'admin_menu', 'adminrole', 'ezmenu', 'ctm',

    // cck
    'token', 'content', 'email', 'fieldgroup', 'filefield', 'getid3', 'filefield_meta', 'link', 'number', 'optionwidgets', 'text', 'nodereference', 'userreference', 'nodereference_url',

    // views
    'views', 'views_export', 'views_ui',

    // date/time
    'date_api', 'date_timezone', 'date', 'date_popup', 'date_repeat', 'calendar', 'calendar_ical', 'jcalendar', 

    // input filters
    'spamspan',

    // other
    'admin_links', 'auto_nodetitle', 'globalredirect', 'imce', 'nice_menus', 'pathauto', 'webform',

    // user interface
    'wysiwyg', 'wwwizard_admin', 'service_links', 'better_formats', 'imce_wysiwyg', 'wwwizard_display',
    
    // multimedia and required modules
    'ffpc', 'transliteration', 'imageapi', 'imageapi_gd', 'imagecache', 'imagecache_ui', 'imagefield', 'views_slideshow', 'mp3player', 'filefieldmp3player', 'video', 'video_upload', 'video_url', 'video_youtube',

    // needed for install profile
    'content_copy', 'wwwizard_views', 'install_profile_api'
  );
  
  return $modules;
}

/**
 * Implementation of hook_profile_task_list.
 */
function wwwizard_profile_task_list() {
  return array(
    'wwwizard-options' => st('Welcoming Websites Wizard Options'),
  );
}


function wwwizard_profile_tasks(&$task, $url) {
  if ($task == 'profile') {
    install_include(wwwizard_profile_modules());
    
    $rids = wwwizard_profile_create_roles_and_perms(); // do this first so we can reference ministers in sermon content type
  
    include_once(drupal_get_path('module', 'install_profile_api') . '/contrib/content_copy.inc');
    wwwizard_profile_create_content_types();
    
    wwwizard_profile_update_perms($rids['editor']); // now give admin and editor content type permis
    
    foreach (array('vocabs', 'menus', 'settings') as $key) {
      wwwizard_profile_create_from_file($key);  
    }
    db_query("update {mp3player_players} set noinfo='yes'"); // don't display track info: mp3player saves this setting outside of the variable table
    
    wwwizard_profile_setup_blocks();
    
    wwwizard_profile_setup_filters($rids);
    
    wwwizard_profile_setup_wysiswyg($rids);
  
    wwwizard_profile_setup_dates();
    
    // it would be better to set up imagecache before content types, as the content type display settings rely on imagecache presets
    // however, this doesn't work if executed before content types are created, so we'll settle
    wwwizard_profile_setup_imagecache();
    
    wwwizard_profile_setup_betterformats($rids);
    
    wwwizard_profile_create_home_page();
    
    // Update the menu router information.
    menu_rebuild();
  
    // clean up unneeded modules
    db_query("update system set status = 0 where name in ('content_copy', 'views_export', 'install_profile_api')");
      
    // allow our modules to have the last say on forms, theme preprocessing, etc
    db_query("update {system} set weight = 10 where name like 'wwwizard_%%'");
    adminrole_update_perms();
    cache_clear_all();
    
    drupal_get_messages();
    
    $task = 'wwwizard-options';
  }
  if ($task == 'wwwizard-options') {
    $output = drupal_get_form('wwwizard_options', $url);

    if (!variable_get('wwwizard_options_submitted', false)) {
      drupal_set_title(st('Welcoming Websites Wizard Options'));
      return $output;      
    }
    else {
      variable_del('wwwizard_options_submitted');
      $task = 'profile-finished';
    }
  }
}

/**
 * Create imagecache presets to support slideshow display
 */
function wwwizard_profile_setup_imagecache() {
  include_once(drupal_get_path('module', 'install_profile_api') . '/contrib/imagecache.inc');

  $preset = array(
    0 => array(
      'action' => 'imagecache_scale',
      'width' => '',
      'height' => '',
      'upscale' => 0,
      'module' => 'imagecache',
    )
  );
  
  $preset[0]['width'] = 200;
  install_imagecache_add_preset('slideshow_image_teaser', $preset);
  
  $preset[0]['width'] = 500;
  install_imagecache_add_preset('slideshow_image_full', $preset);

  $preset[0]['width'] = 150;
  install_imagecache_add_preset('individual_photo_teaser', $preset);
  
  $preset[0]['width'] = 400;
  install_imagecache_add_preset('individual_photo_full', $preset);
}

/**
 * Create roles and permissions.
 */
function wwwizard_profile_create_roles_and_perms() {
  // anonymous and authenticated permissions
  $perms = array('access content', 'access service links', 'access video', 'play video');
  install_add_permissions(1, $perms);
  install_add_permissions(2, $perms);

  // permission to remove from anonymous and authenticated users
  $perms = array('collapse format fieldset by default', 'collapsible format selection', 'show format selection for blocks', 'show format selection for comments', 'show format selection for nodes');
  install_remove_permissions(1, $perms);
  install_remove_permissions(2, $perms);
  
  $roles = array(
    'administrator',
    'editor',
    'blogger',
  );
  
  foreach ($roles as $role) {
    $rids[$role] = install_add_role($role);
  }
  
  // give administrator all permissions
  variable_set('adminrole_adminrole', $rids['administrator']);
  variable_set('wwwizard_editor_role', $rids['editor']);
  adminrole_update_perms();
   
  // now for the editor
  $permissions = array(
    'view advanced help topic',
    "administer blocks",
    "administer menu",
    "administer nodes",
    "delete revisions",
    "revert revisions",
    "view revisions",
    "select different theme",
    "access user profiles",
    "administer permissions",
    "administer users",
    "access webform results",
    "clear webform results",
    "create webforms",
    "edit own webform submissions",
    "edit own webforms",
    "edit webform submissions",
    "edit webforms",
    "create blog entries",
    "delete any blog entry",
    "delete own blog entries",
    "edit any blog entry",
    "edit own blog entries",
    'create video',
    'edit all video nodes',
    'edit own video',
    'administer site configuration',
  );
  install_add_permissions($rids['editor'], $permissions);
  install_add_permissions($rids['administrator'], $permissions);
  
  // let the minister blog
  $permissions = array(
    "create blog entries",
    "delete own blog entries",
    "edit own blog entries",
  );  
  install_add_permissions($rids['blogger'], $permissions);
  
  // make user 1 an administrator
  $admin = user_load(1);
  $roles = $admin->roles;
  $admin_rid = $rids['administrator'];
  $roles[$admin_rid] = $admin_rid;
  user_save($admin, array('roles' => $roles));
  
  return $rids;
}

/**
 * Give the administrator and editor roles all content type permissions
 */
function wwwizard_profile_update_perms($editor_rid) {
  adminrole_update_perms();
  
  $permission = array();
  foreach (node_get_types() as $type => $info) {
    if ($type == 'webform') { continue; }
    
    $permissions[] = "create $type content";
    $permissions[] = "edit any $type content";
    $permissions[] = "delete any $type content";
    $permissions[] = "edit own $type content";
    $permissions[] = "delete own $type content";
  }
  
  foreach ($permissions as $perm) {
    install_add_permissions($editor_rid, $permissions);
  }
}

function wwwizard_profile_create_content_types() {
  $path = drupal_get_path('profile', 'wwwizard') .'/content_types';
  
  $res = opendir($path);
  if (!$res) { return ; }
  
  while ($file = readdir($res)) {
    $matches = array();
    if (preg_match('~^(.*)\.inc$~', $file, $matches)) {
      $file_path = "$path/$file";
      
      $type_name = trim($matches[1]);
      
      if (!in_array($type_name, array('webform', 'blog'))) { // make way for module-created types
        $type_name = '';
      }
      
      install_content_copy_import_from_file($file_path, $type_name);
    }
  }

  if (!file_exists(file_directory_path() . '/pictures')) {
    mkdir(file_directory_path() . '/pictures', 0755);
  }
}

function wwwizard_profile_create_from_file($file) {
  include_once(drupal_get_path('profile', 'wwwizard') ."/$file.inc");
}

function wwwizard_profile_setup_blocks() {
  db_query("update {blocks} set status = 0 where module='user' and delta=1 and theme='garland'");
  db_query("update {blocks} set weight = -9 where module='user' and delta=0 and theme='garland'");
  install_add_block('nice_menus', 1, 'garland', 1, -10, 'left');
  db_query("update {blocks} set title='<none>' where module='nice_menus' and delta=1 and theme='garland'");
  install_add_block('menu', 'menu-site-manage', 'garland', 1, -8, 'left');
  install_add_block('wwwizard_display', 'poweredby', 'garland', 1, 10, 'footer');
}

function wwwizard_profile_setup_filters($rids) {
  db_query("update {filters} set weight=-10 where format=2 and module='filter' and delta=3");
  db_query("insert into {filters} (format, module, delta, weight) values (2, 'spamspan', 0, -9)");
  db_query("update {filters} set weight=-10 where format=2 and module='filter' and delta=2");
  db_query("delete from {filters} where format=2 and module='filter' and delta=1"); // line break converter
  
  // the ",3,4," syntax is an intentional artifact of drupal's method for relating filters to roles
  db_query("update {filter_formats} set roles=',%d,%d,' where format=2", $rids['administrator'], $rids['editor']);
}

function wwwizard_profile_setup_wysiswyg($rids) {
  $settings = array (
    'default' => 1,
    'user_choose' => 0,
    'show_toggle' => 1,
    'theme' => 'advanced',
    'language' => 'en',
    'buttons' => array (
      'default' => array (
        'bold' => 1,
        'italic' => 1,
        'underline' => 1,
        'strikethrough' => 1,
        'justifyleft' => 1,
        'justifycenter' => 1,
        'justifyright' => 1,
        'justifyfull' => 1,
        'bullist' => 1,
        'numlist' => 1,
        'outdent' => 1,
        'indent' => 1,
        'undo' => 1,
        'redo' => 1,
        'link' => 1,
        'unlink' => 1,
        'anchor' => 1,
        'image' => 1,
        'cleanup' => 1,
        'sup' => 1,
        'sub' => 1,
        'blockquote' => 1,
        'code' => 1,
        'cut' => 1,
        'copy' => 1,
        'paste' => 1,
      ),
      'imce' => array (
        'imce' => 1,
      ),
      'wysiwyg' => array (
        'break' => 1,
      ),
    ),
    'toolbar_loc' => 'top',
    'toolbar_align' => 'left',
    'path_loc' => 'bottom',
    'resizing' => 1,
    'verify_html' => 1,
    'preformatted' => 0,
    'convert_fonts_to_spans' => 1,
    'remove_linebreaks' => 0,
    'apply_source_formatting' => 0,
    'paste_auto_cleanup_on_paste' => 1,
    'block_formats' => 'p,address,pre,h2,h3,h4,h5,h6,div',
    'css_setting' => 'none',
    'css_path' => '',
    'css_classes' => '',
  );
  
  $settings = serialize($settings);
  
  if (db_result(db_query("select count(*) as num from {wysiwyg} where format=2"))) {
    db_query("update {wysiwyg} set editor='tinymce', settings='%s where format=2", $settings);
  }
  else {
    db_query("insert into {wysiwyg} (format, editor, settings) values(2, 'tinymce', '%s')", $settings);
  }
  
  variable_set('imce_roles_profiles', array (
    $rids['editor'] => array (
      'weight' => '0',
      'pid' => '2',
    ),
    $rids['administrator'] => array (
      'weight' => '0',
      'pid' => '2',
    ),
    2 => array (
      'weight' => 11,
      'pid' => '0',
    ),
    1 => array (
      'weight' => 12,
      'pid' => '0',
    ),
  ));
  
  foreach ($rids as $role => $rid) {
    foreach (array('node', 'comment') as $type) {
      $format = in_array($role, array('administrator', 'editor')) ? 2 : 1; // full html for admin and editor only
      db_query("UPDATE {better_formats_defaults} SET format=%d, weight=%d WHERE rid=%d AND type='%s'", $format, 0, $rid, $type);
    }  
  }
}

function wwwizard_profile_setup_betterformats($rids) {
  // and now for better formats - make full html the default for editors and admins
  foreach (array('node', 'comment', 'block') as $type) {
    $weight = -25;
    foreach (array($rids['administrator'], $rids['editor']) as $rid) {
      db_query("insert into {better_formats_defaults} (rid, type, format, type_weight, weight) values (%d, '%s', %d, %d, %d)", $rid, $type, 2, 1, $weight);
      $weight++;
    }
  }  
}



function wwwizard_profile_setup_dates() {
  // create "short time" format
  $format = array();
  $format['format'] = "g:ia";
  $format['type'] = 'custom';
  $format['locked'] = 0;
  $format['is_new'] = 1;
  date_format_save($format);
  
  $format_type = array(
    'title' => 'Short time',
    'type' => 'short_time',
    'locked' => 0,
    'is_new' => 1,
  );
  date_format_type_save($format_type);
  variable_set("date_format_short_time", "g:ia");
  
  // make all time formats 12-hour
  variable_set('date_format_long', 'l, F j, Y - g:ia');
  variable_set('date_format_medium', 'D, m/d/Y - g:ia');
  variable_set('date_format_short', 'm/d/Y - g:ia');
}

function wwwizard_profile_create_home_page() {
  $node = new stdClass();
  $node->type = 'page';
  $node->uid = 1;
  $node->title = st('Home');
  $node->body = '<p>'
    . st('Congratulations!  You are ready to begin work on your new Welcoming Websites Wizard-powered site.') . "</p>\n<p>"
    . st('We recommend that you read our !start page for a list of tasks to get you familiar with all of the features available in the Wizard.', array('!start' => l(st('Getting started'), 'help/wwwizard_admin/getting-started'))) . "</p>\n<p>"
    . st('When you are ready to launch your new site, don\'t forget to !edit and welcome visitors to your site.', array('!edit' => l(st('edit this page'), 'node/1/edit'))) . "</p>\n";
  $node->path = 'home';
  $node = node_submit($node);
  node_save($node);
  variable_set('wwwizard_homepage_nid', $node->nid);
}

function wwwizard_profile_create_site_map() {
  include_once(drupal_get_path('profile', 'wwwizard') ."/site_map.inc");
  
  $mlids['<root>'] = 0; // root of the sitemap tree
  $num_children['<root>'] = 0;
  
  foreach ($nodes as $node) {
    $n = new stdClass();
    $n->type = $node['type'];
    $n->uid = 1;
    $n->title = t($node['title']);
    $n->body = '';
    
    $parent_path = $node['parent'];
    
    $n->path = $node['path'];
    if ($parent_path != '<root>') {
      $prefix = $path[$parent_path];
      $n->path = "$prefix/$n->path"; // e.g. visit-us/getting-here
    }
    
    $n = node_submit($n);
    node_save($n);

    if ($parent_path) {
      $item = array(
        'link_title' => $n->title,
        'parent' => 'menu-sitemap:' . $mlids[$parent_path],
        'weight' => intval($num_children[$parent_path]),
        'menu_name' => 'menu-sitemap',
        'link_path' => "node/" . $n->nid,
        'plid' => $mlids[$parent_path],
      );
      menu_link_save($item);
    }
  
    $my_path = $node['path'];
    $nids[$my_path] = $n->nid; // save nids so that we can refer back to them when creating children
    $mlids[$my_path] = $item['mlid'];
    $num_children[$my_path] = 0;
    $num_children[$parent_path]++;
    $path[$my_path] = $n->path;
  }
}

/**
 * Form which allows user to choose:
 * - Whether to create default WWWizard site map
 * - Description of site's time zone, as required by date module
 * - Username and password for editor user, to be used in day-to-day editing
 */
function wwwizard_options($form_state, $url) {
  $form = array();
  $form['#redirect'] = false;
  $form['#action'] = $url;
  
  // site map
  $form['site_map'] = array(
    '#title' => st('Pre-load default site map'),
    '#type' => 'checkbox',
    '#description' => st('Welcoming Websites Wizard includes a default site map, which can help you get started in building your site.  The site map is just a starting point, to illustrate what you can do with your new site; pages in the site may be edited, deleted, or re-arranged.  Leaving this checkbox unchecked means you will start out with a blank site.'),
    '#weight' => -10,
  );
  
  // timezone
  $timezone = variable_get('date_default_timezone_name', NULL);
  $form['date_default_timezone_name'] = array (
    '#type' => 'select',
    '#title' => t('Default time zone name'),
    '#default_value' => $timezone,
    '#options' => date_timezone_names(FALSE, TRUE), // Force an update before setting a site default.
    '#description' => t('Select the name of the default site time zone. If in doubt, choose the timezone that is closest to your location which has the same rules for daylight saving time.'),
    '#weight' => 0,
    '#offset' => variable_get('date_default_timezone', 0),
  );
  
  // editor account
  if (variable_get('wwwizard_editor_role', 0)) {
    _user_password_dynamic_validation();
    
    // form elements below are based on elements from install_configure_form
    $form['editor_account'] = array(
      '#type' => 'fieldset',
      '#title' => st('Editor account'),
      '#collapsible' => FALSE,
      '#weight' => 5,
    );
    
    $form['editor_account']['account']['#tree'] = TRUE;
    $form['editor_account']['markup'] = array(
      '#value' => '<p class="description">'. st('The editor account is intended for day-to-day site editing purposes.  This account can be provided to someone who may not be interested in all site administration tasks, but may be responsible for many content creation and editing responsibilities.  Additional editor accounts may be created once the Wizard has been installed.') .'</p>',
      '#weight' => -10,
    );
  
    $editor_rid = variable_get('wwwizard_editor_role', 0);
    $form['editor_account']['account']['roles'] = array(
      '#type' => 'value',
      '#value' => array(
        2 => 2, // authenticated user
        $editor_rid => $editor_rid, // editor
      ),
    );
    
    $form['editor_account']['account']['name'] = array(
      '#type' => 'textfield',
      '#title' => st('Username'),
      '#maxlength' => USERNAME_MAX_LENGTH,
      '#description' => st('Spaces are allowed; punctuation is not allowed except for periods, hyphens, and underscores.'),
      '#required' => TRUE,
      '#weight' => -8,
    );

    $form['editor_account']['account']['mail'] = array(
      '#type' => 'textfield',
      '#title' => st('E-mail address'),
      '#maxlength' => EMAIL_MAX_LENGTH,
      '#description' => st('The e-mail address for the editor; it should be different from the e-mail address for the administrator.'),
      '#required' => TRUE,
      '#weight' => -5,
    );
    $form['editor_account']['account']['pass'] = array(
      '#type' => 'password_confirm',
      '#required' => TRUE,
      '#size' => 25,
      '#weight' => 0,
    );
  }
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => st('Continue'),
    '#weight' => 10,
  );
  
  return $form;
}

/**
 * Validate editor username and email address.
 * Similar to install_configure_form_validate().
 */
function wwwizard_options_validate($form, &$form_state) {
  if ($error = user_validate_name($form_state['values']['account']['name'])) {
    form_error($form['editor_account']['account']['name'], $error);
  }
  if ($error = user_validate_mail($form_state['values']['account']['mail'])) {
    form_error($form['editor_account']['account']['mail'], $error);
  }
}

/**
 * Act on options specified in options form.
 */
function wwwizard_options_submit($form, &$form_state) {
  global $user;
  
  if ($form_state['values']['site_map']) {
    wwwizard_profile_create_site_map();
  }
  
  // create editor account
  if ($form_state['values']['account']['name']) {
    $account = new stdClass();
    $data = $form_state['values']['account'];
    $data['status'] = 1;
    
    user_save($account, $data);
  }
  
  // set timezone description for date module
  variable_set('date_default_timezone_name', $form_state['values']['date_default_timezone_name']);
  variable_set('wwwizard_options_submitted', true);
}